;%include "x-face.inc"

; -----------------------------------------------------------------------------

%define MSG_SET_XFACE 	WM_USER+100
%define MSG_CLEAR_XFACE WM_USER+101

%define timer_freq	100
%define pos_save_waits	20

; -----------------------------------------------------------------------------

[section .bss]
; ptr to headers that identify the current post. used to determine if this
; routine is called with headers from a different posting (so that we can
; remove the X-Face from the display). this just works, although I've only
; checked it empirically.
current_post	resd 1

; thread id (not the handle) for the thread managing the window
thread_id	resd 1

temp		resb 14 + 100   ; let's hope our error text doesn't get any longer
				; than this

grav_hwnd	resd 1

hidden		resb 1

counter		resd 1

x		resd 1
y		resd 1

rect		resb RECT_size

__SECT__

; -----------------------------------------------------------------------------

[section .data]

disabled	db 0
initialized	db 0

error_text	db "Error %d: %s", 0

dib_error	db "CreateDIBSection failed", 0

__SECT__

; -----------------------------------------------------------------------------

%macro checkerror 2
; check for error
; %1 statement to set flags, i.e. "test eax, eax" or something
; %2 condition code - if this is true an error is reported with the line number
;    set to the line before the line containing "checkerror"
		%1
		j%-2	short %%ok
		mov	esi, __LINE__ - 1
		jmp	show_error
%%ok:
%endmacro

show_error:
		invoke	GetLastError
		invoke	FormatMessageA, dword FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_ALLOCATE_BUFFER, 0, eax, 0, dword err_msg_ptr, 0, 0
		mov	ebx, temp
		invoke	wsprintfA, ebx, dword error_text, esi, dword [err_msg_ptr]
		add	esp, 4 * 4
		invoke	MessageBoxA, byte 0, ebx, dword title, byte 0

		; no need to LocalFree, we won't come back here again anyway

		mov	byte [disabled], 1

		xor	eax, eax
		ret	4


global patch
patch:
		; [esp+4]: ptr to article all headers, ASCIIZ
                ; Original multiline headers have been put on one line by Gravity!

		cmp	byte [disabled], 1
		je	.error_return

		cmp	byte [initialized], 1
		je	short .already_initialized

                ; create thread which will manage the window
		invoke	CreateThread, 0, dword 4096, dword thread_proc, 0, 0, dword thread_id
		checkerror {test eax, eax}, z

                ; give our dialog time to setup its message queue so that
                ; PostMessage won't fail (if the first post displayed
                ; has a X-Face, we do PostMessage real soon down here,
                ; and PM might fail because the dialog isn't created yet)
		invoke	Sleep, 0

		mov	byte [initialized], 1

.already_initialized:
		mov	esi, [esp+4]

		mov	ebx, esi		; save beginning of headers
.h:
		mov	edx, esi
.scan_eol:
		cmp	byte [esi], 0
		je	short .eol
                cmp     byte [esi], CR
		je	short .eol
                cmp     byte [esi], LF
		je	short .eol
		inc	esi
		jmp	short .scan_eol
.eol:
		mov	eax, esi
		sub	eax, edx		; eax is length of header line
		lea	ecx, [eax-7]		; ecx: length of X-Face, without "X-Face: " (-8) and with Z (+1)
		mov	edi, eax
		cmp	eax, 9			; is the header line shorter than 9 bytes?
		jl	.next_header

		; edx: beginning of header line
		; esi: 1 past end of header line
		; eax: length of header line	; not used any further down
		; ecx: length of X-Face (if any)

		; scan header line for "X-Face: "

		push	edx
		push	ecx

		; convert the first 8 chars to lowercase and store those in
                ; temp (so that we catch all combinations such as "X-Face: ",
                ; "x-face: ", "X-face: ", etc.
		push	byte 8
		pop	ecx
		mov	edi, temp
.to_lower:
		mov	al, byte [edx]
		inc	edx
		cmp	al, "A"
		jl	short .next_char
		cmp	al, "Z"
		ja	short .next_char
		add	al, 32
.next_char:
		mov	[edi], al
		inc	edi
		loop	.to_lower

		pop	ecx
		pop	edx

		cmp	dword [temp], "x-fa"
		jne	.next_header
		cmp	dword [temp+4], "ce: "
		jne	short .next_header

                ; we got a X-Face!

		mov	dword [current_post], ebx

                cmp     ecx, 4096               ; X-Face max one page in size
		ja	.quit

                ; pass X-Face to window manager thread

		; alloc one page (4096 bytes)
		push	ecx
		push	edx
 		invoke	VirtualAlloc, byte 0, eax, dword MEM_COMMIT, byte PAGE_READWRITE
		mov	edi, eax
		pop	edx
		pop	ecx
		checkerror {test eax, eax}, z

		mov	bh, byte [esi]		; save byte from Gravity buffer
		mov	byte [esi], 0		; make header line zero terminated

		add	edx, 8			; skip "X-Face: "

.copy_face:
		mov	bl, byte [edx]
		mov	byte [eax], bl
		inc	edx
		inc	eax
		loop	.copy_face

		invoke	PostMessageA, dword [hwnd], dword MSG_SET_XFACE, edi, eax
		test	eax, eax
		jnz	short .post_ok

		; free buffer - dialog won't receive message
		invoke	VirtualFree, edi, byte 0, dword MEM_RELEASE

.post_ok:
		mov	byte [esi], bh		; repair Gravity buffer

		jmp	short .got_face		; don't scan the headers any further

.next_header:
		cmp	byte [esi], 0
		je	short .no_face

		xor	eax, eax

		cmp	byte [esi], CR
		sete	al
		add	esi, eax

		cmp	byte [esi], LF
		sete	al
		add	esi, eax

		cmp	byte [esi], CR
		sete	al
		add	esi, eax

		cmp	byte [esi], 0
		je	short .no_face

		; esi: ptr to beginning of next header
		mov	edx, esi
		jmp	.scan_eol

.no_face:
		cmp	ebx, dword [current_post]
		je	.same_posting

		; notify dialog to remove current X-Face from screen
                ; as the posting has changed and does not have a X-Face

		invoke	PostMessageA, dword [hwnd], dword MSG_CLEAR_XFACE, eax, eax

.got_face:
.same_posting:
.quit:

                xor     eax, eax
                inc     eax
                ret	4

.error_return:
		xor	eax, eax
		ret	4


; -----------------------------------------------------------------------------

[section .data]

grav_classname	db	"MicroPlanetNewsfra"
me		db	"me", 0

ini_name	db	"x-face-patch.ini", 0

key_name_x	db "x", 0
key_name_y	db "y", 0
key_name_sx	db "sx", 0
key_name_sy	db "sy", 0

fmt_str		db "%d", 0

window_title	db "X-Face", 0

__SECT__

thread_proc:
		invoke	DialogBoxParamA, dword [instance], IDD_FACEDIALOG, 0, dword face_dialog_proc, eax
		ret	4

face_dialog_proc:

%define _hwnd	ebp+8
%define msg	ebp+12
%define wParam	ebp+16
%define lParam	ebp+20

		push	ebp
		mov	ebp, esp

		push	ebx
		push	esi
		push	edi

		mov	eax, [msg]

		cmp	eax, WM_INITDIALOG
		jne	.l0

		mov	eax, [_hwnd]
		mov	[hwnd], eax
		mov	ebx, eax

		; start building args for SetWindowPos()

		push	SWP_NOZORDER		; uFlags

		invoke	GetSystemMetrics, byte SM_CYFIXEDFRAME
		lea	esi, [eax * 2 + 48 + 1]
		invoke	GetSystemMetrics, byte SM_CYSMSIZE
		add	esi, eax
		push	esi			; cy

		invoke	GetSystemMetrics, byte SM_CXFIXEDFRAME
		lea	eax, [eax * 2 + 48]
		push	eax			; cx

		xor	esi, esi
		xor	edi, edi

		invoke	GetPrivateProfileIntA, dword me, dword key_name_sy, byte 0, dword ini_name
		push	eax
		invoke	GetSystemMetrics, byte SM_CYSCREEN
		pop	ecx
		cmp	eax, ecx
		jne	short .topleft

		invoke	GetPrivateProfileIntA, dword me, dword key_name_sx, byte 0, dword ini_name
		push	eax
		invoke	GetSystemMetrics, byte SM_CXSCREEN
		pop	ecx
		cmp	eax, ecx
		jne	short .topleft

		invoke	GetPrivateProfileIntA, dword me, dword key_name_y, byte 0, dword ini_name
		mov	esi, eax
		invoke	GetPrivateProfileIntA, dword me, dword key_name_x, byte 0, dword ini_name
		mov	edi, eax

.topleft:
		push	esi			; y
		push	edi			; x

		push	0			; hwndInsertAfter (ignored)
		push	ebx			; hwnd
		invoke	SetWindowPos

		; setup timer

		mov	dword [counter], pos_save_waits

		invoke	SetTimer, ebx, 0, dword timer_freq, 0	; save window pos every timer_freq milliseconds

		; create DIB

		invoke	CreateDIBSection, byte 0, dword bitmap_info_header, byte 0, dword dib_pixels, byte 0, byte 0
		test	eax, eax
		jnz	short .ok

		invoke	MessageBoxA, byte 0, dword dib_error, dword title, byte 0
		invoke	EndDialog, dword [hwnd], eax

		jmp	.exit

.ok:
		mov	dword [dib_handle], eax

		invoke	GetDlgItem, ebx, dword IDC_FACEIMAGE
		invoke	SendMessageA, eax, dword STM_SETIMAGE, IMAGE_BITMAP, dword [dib_handle]

		invoke	FindWindowA, dword grav_classname, 0
		invoke	SetForegroundWindow, eax

		jmp	.exit

.l0:
		cmp	eax, MSG_SET_XFACE
		jne	short .l1

		; display the X-Face wParam points to

		push	2
		push	dword [dib_pixels]
		push	dword [wParam]
		call	decode_face
		mov	ebx, eax

		invoke	RedrawWindow, dword [_hwnd], 0, 0, byte RDW_INVALIDATE

		invoke	VirtualFree, dword [wParam], 0, dword MEM_RELEASE

		test	ebx, ebx
		jns	short .ok1

		invoke	MessageBeep, 0
		jmp	short .clear
.ok1:

		jmp	.exit

.l1:
		cmp	eax, MSG_CLEAR_XFACE
		jne	short .l2
.clear:

		; remove X-Face

		xor	eax, eax
		mov	edi, dword [dib_pixels]
		mov	ecx, 8 * 48		; 8, not 6, because of DIB alignment
		rep	stosb

		invoke	RedrawWindow, dword [_hwnd], byte 0, byte 0, byte RDW_INVALIDATE

%if 0
		invoke	GetDlgItem, dword [_hwnd], dword IDC_FACEIMAGE
		invoke	SendMessageA, eax, dword STM_SETIMAGE, IMAGE_BITMAP, 0
		invoke	RedrawWindow, dword [_hwnd], byte 0, 0, byte RDW_INVALIDATE
%endif

		jmp	.exit

.l2:
		cmp	eax, WM_TIMER
		jne	.l3

		dec	byte [counter]
		jnz	.dont_save

		mov	dword [counter], pos_save_waits

		; look if we need to save the window position

		invoke	GetWindowRect, dword [_hwnd], dword rect

		mov	eax, dword [rect+RECT.left]
		mov	ebx, dword [rect+RECT.top]

		cmp	eax, dword [x]
		je	.dont_save
		jmp	.save

		cmp	ebx, dword [y]
		je	.dont_save
.save:
		; save the window coords

		mov	dword [x], eax
		mov	dword [y], ebx

		mov	edi, temp
		mov	esi, ini_name

		invoke	wsprintfA, edi, dword fmt_str, eax
		add	esp, 3*4

		mov	ecx, key_name_x
		invoke	WritePrivateProfileStringA, dword me, ecx, edi, esi

		mov	eax, dword [y]

		invoke	wsprintfA, edi, dword fmt_str, eax
		add	esp, 3*4

		mov	ecx, key_name_y
		invoke	WritePrivateProfileStringA, dword me, ecx, edi, esi

		invoke	GetSystemMetrics, SM_CXSCREEN

		invoke	wsprintfA, edi, dword fmt_str, eax
		add	esp, 3*4

		mov	ecx, key_name_sx
		invoke	WritePrivateProfileStringA, dword me, ecx, edi, esi

		invoke	GetSystemMetrics, SM_CYSCREEN

		invoke	wsprintfA, edi, dword fmt_str, eax
		add	esp, 3*4

		mov	ecx, key_name_sy
		invoke	WritePrivateProfileStringA, dword me, ecx, edi, esi

.dont_save:
		invoke	FindWindowA, dword grav_classname, 0
		test	eax, eax
		jz	short .inactive

		mov	ebx, eax

		invoke	GetForegroundWindow
		cmp	eax, dword [_hwnd]
		je	short .active
		cmp	eax, ebx
		je	short .active

.inactive:
		invoke	ShowWindow, dword [_hwnd], SW_HIDE
		mov	byte [hidden], 1

		jmp	short .end
.active:

		cmp	byte [hidden], 1
		jne	short .end

		invoke	ShowWindow, dword [_hwnd], SW_SHOW
		mov	byte [hidden], 0

.end:

		jmp	short .exit
.l3:

		xor	eax, eax
.exit:
		pop	edi
		pop	esi
		pop	ebx

		leave
		ret	4 * 4